import { Controller } from '@hotwired/stimulus';
import debounce from 'debounce';

export default class extends Controller {
  static targets = ['mySelection', 'inputSearch', 'tagsetRadio', 'submitButton', 'loader', 'completeness', 'fileInput'];

  connect() {
    this.addRadioListeners();
    this.inputSearchTarget.addEventListener('keyup', debounce(this.handleKeyup.bind(this), 300));

    if (this.hasFileInputTarget) {
      this.fileInputTarget.addEventListener('change', () => this.updateButtonVisibility());
    }
  }

  disconnect() {
    this.removeRadioListeners();
    this.inputSearchTarget.removeEventListener('keyup', this.handleKeyup);
    if (this.hasFileInputTarget) {
      this.fileInputTarget.removeEventListener('change', () => this.updateButtonVisibility());
    }
  }

  addRadioListeners() {
    this.tagsetRadioTargets.forEach(radio => {
      radio.addEventListener('change', () => this.updateButtonVisibility());
    });
  }

  removeRadioListeners() {
    this.tagsetRadioTargets.forEach(radio => {
      radio.removeEventListener('change', () => this.updateButtonVisibility());
    });
  }

  updateButtonVisibility() {
    const shouldShowButton = this.hasFileInputTarget ? this.fileInputTarget.files.length > 0 : this.tagsetRadioTargets.some(radio => radio.checked);

    this.submitButtonTarget.classList.toggle('hidden', !shouldShowButton);
  }

  handleKeyup(event) {
    const query = event.target.value;

    if (query.length > 2) {
      this.search(query);
    }
  }

  search(query) {
    const url = `/spirit_hub/tagsets/search_directories?query=${encodeURIComponent(query)}`;
    this.loaderTarget.classList.remove('hidden');

    fetch(url, { headers: { Accept: 'text/vnd.turbo-stream.html' } })
      .then(response => response.text())
      .then(html => {
        this.removeRadioListeners();
        Turbo.renderStreamMessage(html);
        requestAnimationFrame(() => {
          this.addRadioListeners();
          this.updateButtonVisibility();
          this.loaderTarget.classList.add('hidden');
        });
      })
      .catch(error => console.error('Error:', error));
  }

  tagsetChanged(event) {
    const selectedValue = event.target.value;
    const isChecked = event.target.checked;
    const matchingDivs = this.element.querySelectorAll(`div[data-tagset-id="${selectedValue}"]`);

    this.completenessTargets.forEach(div => div.classList.add('hidden'));

    if (isChecked) {
      matchingDivs.forEach(div => div.classList.remove('hidden'));
    }
  }
}
