# frozen_string_literal: true

module SpiritHub
  class DirectoriesController < ApplicationController
    before_action :find_models, only: %i[index sort]

    include Datatablable

    def index
      skip_authorization
      skip_policy_scope

      respond_to(&:html)
    end

    def sort
      skip_authorization
      skip_policy_scope

      render partial: table_partial
    end

    def add_doc_to_completeness_form
      skip_authorization
      skip_policy_scope

      @completeness = SpiritHub::Completeness.find(params[:completeness_id])
      @tagsets = [@completeness.tagset]

      render turbo_stream: [
        turbo_stream.update(:modal, partial: 'spirit_hub/tagsets/add_doc_form', locals: { form_url: add_doc_to_completeness_spirit_hub_directories_path })
      ]
    end

    def add_doc_to_completeness # rubocop:disable Metrics/AbcSize,Metrics/PerceivedComplexity
      skip_authorization
      skip_policy_scope

      file = params[:file]
      doc = Doc.create(responsible: @user.id, title: file.original_filename)
      doc.blob = file.read
      doc.update(type: 'file')
      tagset_doc = Tagset.create!(doc_ids: [doc.id], responsible: @user.id, type: 'document', title: file.original_filename)

      if params[:tagset_id].present?
        if (tagset = Tagset.find_by(id: params[:tagset_id]))
          tagset.update(tagset_ids: tagset.tagset_ids.to_a + [tagset_doc.id])
        elsif (completeness = Completeness.find_by(id: params[:tagset_id]))
          completeness.tagset.update(tagset_ids: completeness.tagset.tagset_ids.to_a + [tagset_doc.id])
          completeness.update(object_ids: completeness.object_ids.to_a + [tagset_doc.id])
        end
        if (new_state = tagset_doc.cycle&.states&.find_by(key: :share))
          tagset_doc.update(state: new_state)
        end
      end
      set_user_favorites
      set_columns
      flash.now[:notice] = 'Documents ajouté'

      partial = params[:parent_ids] && params[:parent_ids].split(',').length.positive? ? 'spirit_hub/directories/sub_row' : 'spirit_hub/directories/row'

      tagset_doc.reload
      turbo_actions = [
        turbo_stream.replace("completeness_#{completeness.id}", partial:, locals: { model: tagset_doc, parent_ids: params[:parent_ids].split(','), show: true }),
        turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
        turbo_stream.update("tagset_#{completeness.tagset.id}_current_state_percent", partial: 'spirit_hub/common/current_state_percent',
                                                                                      locals: { model: completeness.tagset.reload }),
        turbo_stream.update("tagset_#{completeness.tagset.id}_global_avancement", partial: 'spirit_hub/common/percent_completion',
                                                                                  locals: { percent: completeness.tagset.content_completeness })
      ]

      render turbo_stream: turbo_actions
    end

    def remove # rubocop:disable Metrics/AbcSize
      skip_authorization
      skip_policy_scope

      @model = Tagset.find(params[:id])

      @completeness = SpiritHub::Completeness.find_by(object_ids: @model.id)
      @model.current_user_id = @user.id
      @model.destroy
      parent_ids = []
      parent_ids = params[:parent_ids].split(',') if params[:parent_ids].present?
      tagset = Tagset.find_by(id: parent_ids.last) if parent_ids.length.positive?
      directory = @model.parent

      flash.now[:notice] = 'Document supprimé'

      turbo_actions = [
        turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
      ]

      if tagset
        tagset.reload
        turbo_actions << turbo_stream.update("tagset_#{tagset.id}_current_state_percent", partial: 'spirit_hub/common/current_state_percent', locals: { model: tagset.reload })
      end

      if directory
        directory.reload
        turbo_actions << turbo_stream.update("tagset_#{directory.id}_global_avancement", partial: 'spirit_hub/common/percent_completion',
                                                                                         locals: { percent: directory.content_completeness })
      end

      turbo_actions << if @completeness
                         set_user_favorites
                         set_columns

                         turbo_stream.replace("tagset_#{params[:id]}", partial: 'spirit_hub/directories/completeness',
                                                                       locals: { model: @completeness.reload, parent_ids:, show: true })
                       else
                         turbo_stream.remove("tagset_#{params[:id]}")
                       end
      render turbo_stream: turbo_actions
    end

    def bulk_delete
      skip_authorization
      skip_policy_scope

      ids = params[:ids].split(',')

      Tagset.where(:id.in => ids).each do |tagset|
        tagset.current_user_id = @user.id
        tagset.destroy
      end

      flash.now[:notice] = 'Documents supprimés'

      render turbo_stream: [
        *ids.map { |id| turbo_stream.remove("tagset_#{id}") },
        turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
        turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -ids.length })

      ]
    end

    private

    def default_columns
      {
        type: { label: 'Type', sortable: true },
        cleaned_main_key: { label: 'Libellé', sortable: true },
        doc_file_size: { label: 'Poids', sortable: true },
        updated_at: { label: 'Dernière modification', sortable: true },
        last_current_user_name: { label: 'Provenance', sortable: true },
        informed: { label: 'Partagé', sortable: false },
        current_state: { label: 'Statut', sortable: true }
      }.freeze
    end

    def find_models
      @directory = Tagset.find(params[:id])
      @logs = @directory.logs.order(created_at: :desc)
      scope = Tagset.where(:id.in => @directory.tagset_ids.to_a)
      @models = find_models_page(scope, default_sort_by: 'cleaned_main_key')
    end
  end
end
