# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(Tagset, type: :model) do
  describe 'add_default_cycle' do
    let!(:default_cycle) { create(:cycle, key: 'hubdoc') }

    it 'adds default cycle' do
      tagset = create(:tagset)
      expect(tagset.cycle_id).to eq(default_cycle.id)
    end

    it 'no error when no default cycle' do
      default_cycle.destroy
      tagset = create(:tagset)
      expect(tagset.cycle_id).to be_nil
    end
  end

  describe 'update_operation_title' do
    let!(:operation) { create(:operation, title: 'Operation test') }

    it 'updates operation title' do
      tagset = create(:tagset, operation:)
      expect(tagset.operation_title).to eq('Operation test')

      operation.update(title: 'Operation test 2')
      tagset.reload

      expect(tagset.operation_title).to eq('Operation test 2')
    end
  end

  describe 'update_current_state' do
    let!(:tagset) { create(:tagset) }

    subject { tagset.reload.current_state }

    it 'default' do
      expect(subject).to eq('Complet')
    end

    it 'content_completeness 0' do
      tagset.update(content_completeness: 0)
      expect(subject).to eq('À completer')
    end

    it 'content_completeness 50' do
      tagset.update(content_completeness: 50)
      expect(subject).to eq('À completer')
    end

    it 'content_completeness 100' do
      tagset.update(content_completeness: 100)
      expect(subject).to eq('Complet')
    end
  end

  describe 'update_informed_fields' do
    let!(:user1) { create(:user, first_name: 'User', last_name: '1') }
    let!(:user2) { create(:user, first_name: 'User', last_name: '2') }

    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.informed_names).to eq([])
      expect(tagset.informed_picture_urls).to eq([])
    end

    it 'add user' do
      tagset.update(informed: [user1.id.to_s])
      expect(tagset.informed_names).to eq(['User 1'])
      expect(tagset.informed_picture_urls).to eq([user1.reload.picture_url])
    end

    it 'add 2 users and remove them' do
      tagset.update(informed: [user1.id.to_s, user2.id.to_s])
      expect(tagset.informed_names).to eq(['User 1', 'User 2'])
      expect(tagset.informed_picture_urls).to eq([user1.reload.picture_url, user2.reload.picture_url])

      tagset.update(informed: [])
      expect(tagset.informed_names).to eq([])
      expect(tagset.informed_picture_urls).to eq([])
    end
  end

  describe 'add_hubdoc_label' do
    let!(:tagset) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.labels).to include('hubdoc')
    end

    it 'dont add label if already present' do
      tagset.update(labels: %w[hubdoc other])
      expect(tagset.reload.labels).to eq(%w[hubdoc other])
    end
  end

  describe 'update_children_parent_ids' do
    let!(:tagset) { create(:tagset) }
    let!(:child1) { create(:tagset) }
    let!(:child2) { create(:tagset) }

    it 'default' do
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end

    it 'add child' do
      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
    end

    it 'add 2 children and remove one' do
      tagset.update(tagset_ids: [child1.id.to_s, child2.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([tagset.id])

      tagset.update(tagset_ids: [child1.id.to_s])
      expect(tagset.reload.parent_tagset_ids).to eq([])
      expect(child1.reload.parent_tagset_ids).to eq([tagset.id])
      expect(child2.reload.parent_tagset_ids).to eq([])
    end
  end

  describe 'update_completeness' do
    let!(:user) { create(:user) }
    let!(:tagset1) { create(:tagset) }
    let!(:tagset2) { create(:tagset) }
    let!(:completeness) { create(:completeness, tagset_id: tagset2.id, object_ids: [tagset1.id]) }

    it 'default' do
      expect(completeness.reload.object_ids).to eq([tagset1.id])
    end

    it 'when destroy tagset' do
      tagset1.current_user_id = user.id
      tagset1.destroy
      expect(completeness.reload.object_ids).to eq([])
    end
  end

  describe 'share_to_users' do
    let!(:tagset) { create(:tagset_doc, :with_file) }
    let!(:user1) { create(:user) }
    let!(:user2) { create(:user) }

    it 'shares to users' do
      expect { tagset.share_to_users([user1.id.to_s], [user2.id.to_s], [], user1) }.to change(Upload, :count).by(1)
      expect(tagset.reload.informed).to eq([user1.id, user2.id])
      expect(Log.count).to eq(1)
    end
  end
end
